@@ -15,8 +15,8 @@ from os.path import join
 from sqlalchemy.exc import IntegrityError, DatabaseError
 from sqlalchemy.orm import Session
 
-from medic.orm import Medline, Section, Author, Descriptor, Qualifier, Database, Identifier, \
-        Chemical, Keyword, PublicationType
+from medic.orm import Citation, Section, Abstract, Author, Descriptor, \
+    Qualifier, Database, Identifier, Chemical, Keyword, PublicationType
 from medic.parser import MedlineXMLParser, PubMedXMLParser, Parser
 from medic.web import Download
 
@@ -33,10 +33,10 @@ def update(session: Session, files_or_pmids: iter, uniq: bool) -> bool:
     _add(session, files_or_pmids, lambda i: session.merge(i), uniq)
 
 
-def select(session: Session, pmids: list([int])) -> iter([Medline]):
-    "Return an iterator over all `Medline` records for a list of *PMIDs*."
+def select(session: Session, pmids: list([int])) -> iter([Citation]):
+    "Return an iterator over all `Citation` records for a list of *PMIDs*."
     count = 0
-    for record in session.query(Medline).filter(Medline.pmid.in_(pmids)):
+    for record in session.query(Citation).filter(Citation.pmid.in_(pmids)):
         count += 1
         yield record
     logger.info("retrieved %i records", count)
@@ -44,7 +44,7 @@ def select(session: Session, pmids: list([int])) -> iter([Medline]):
 
 def delete(session: Session, pmids: list([int])) -> bool:
     "Delete all records for a list of *PMIDs*."
-    count = session.query(Medline).filter(Medline.pmid.in_(pmids)).delete(
+    count = session.query(Citation).filter(Citation.pmid.in_(pmids)).delete(
         synchronize_session=False
     )
     session.commit()
@@ -66,16 +66,28 @@ def dump(files: iter, output_dir: str, unique: bool, update: bool):
                    added to the list of PMIDs for deletion
     """
     out_stream = {
-        Medline.__tablename__: open(join(output_dir, "records.tab"), "wt"),
-        Section.__tablename__: open(join(output_dir, "sections.tab"), "wt"),
-        Descriptor.__tablename__: open(join(output_dir, "descriptors.tab"), "wt"),
-        Qualifier.__tablename__: open(join(output_dir, "qualifiers.tab"), "wt"),
-        Author.__tablename__: open(join(output_dir, "authors.tab"), "wt"),
-        Identifier.__tablename__: open(join(output_dir, "identifiers.tab"), "wt"),
-        Database.__tablename__: open(join(output_dir, "databases.tab"), "wt"),
-        PublicationType.__tablename__: open(join(output_dir, "publication_types.tab"), "wt"),
-        Chemical.__tablename__: open(join(output_dir, "chemicals.tab"), "wt"),
-        Keyword.__tablename__: open(join(output_dir, "keywords.tab"), "wt"),
+        Citation.__tablename__:
+        open(join(output_dir, "citations.tab"), "wt"),
+        Abstract.__tablename__:
+        open(join(output_dir, "abstracts.tab"), "wt"),
+        Section.__tablename__:
+        open(join(output_dir, "sections.tab"), "wt"),
+        Descriptor.__tablename__:
+        open(join(output_dir, "descriptors.tab"), "wt"),
+        Qualifier.__tablename__:
+        open(join(output_dir, "qualifiers.tab"), "wt"),
+        Author.__tablename__:
+        open(join(output_dir, "authors.tab"), "wt"),
+        Identifier.__tablename__:
+        open(join(output_dir, "identifiers.tab"), "wt"),
+        Database.__tablename__:
+        open(join(output_dir, "databases.tab"), "wt"),
+        PublicationType.__tablename__:
+        open(join(output_dir, "publication_types.tab"), "wt"),
+        Chemical.__tablename__:
+        open(join(output_dir, "chemicals.tab"), "wt"),
+        Keyword.__tablename__:
+        open(join(output_dir, "keywords.tab"), "wt"),
         'delete': open(join(output_dir, "delete.txt"), "wt"),
     }
     count = 0
@@ -110,7 +122,7 @@ def _dump(in_stream, out_stream: dict, parser: Parser, update: bool) -> int:
         else:
             out_stream[i.__tablename__].write(str(i))
 
-            if i.__tablename__ == Medline.__tablename__:
+            if i.__tablename__ == Citation.__tablename__:
                 count += 1
 
                 if update:
@@ -122,14 +134,17 @@ def _dump(in_stream, out_stream: dict, parser: Parser, update: bool) -> int:
 def _add(session: Session, files_or_pmids: iter, dbHandle, unique: bool=True):
     pmids = []
     count = 0
-    initial = session.query(Medline).count() if logger.isEnabledFor(logging.INFO) else 0
+    initial = session.query(Citation).count() if \
+        logger.isEnabledFor(logging.INFO) else 0
 
     try:
         for arg in files_or_pmids:
             try:
                 pmids.append(int(arg))
             except ValueError:
-                count += _streamInstances(session, dbHandle, _fromFile(arg, unique))
+                count += _streamInstances(
+                    session, dbHandle, _fromFile(arg, unique)
+                )
 
         if len(pmids):
             count += _downloadAll(session, dbHandle, pmids, unique)
@@ -137,12 +152,12 @@ def _add(session: Session, files_or_pmids: iter, dbHandle, unique: bool=True):
         session.commit()
 
         if logger.isEnabledFor(logging.INFO):
-            final = session.query(Medline).count()
+            final = session.query(Citation).count()
             logger.info('parsed %i entities (records before/after: %i/%i)',
                         count, initial, final)
         return True
     except IntegrityError:
-        logger.exception('DB integrity violated')
+        logger.exception('DB integrity violated (duplicate records?)')
         session.rollback()
         return False
     except DatabaseError:
@@ -204,9 +219,9 @@ def _collectCitation(stream: iter) -> iter:
 
 def _handleCitation(handle, instances: list):
     "Handle a list of instances representing a citation."
-    # handle the Medline object first:
+    # handle the Citation object first:
     for idx in range(len(instances) - 1, -1, -1):
-        if isinstance(instances[idx], Medline):
+        if isinstance(instances[idx], Citation):
             logger.debug('handling PMID %s', instances[idx].pmid)
             handle(instances.pop(idx))
             break
@@ -218,7 +233,8 @@ def _handleCitation(handle, instances: list):
     return 1
 
 
-def _downloadAll(session: Session, dbHandle, pmids: list, unique: bool=True) -> int:
+def _downloadAll(session: Session, dbHandle,
+                 pmids: list, unique: bool=True) -> int:
     """
     Download PubMed XML for a list of PMIDs (integers), parse the streams,
     and send the ORM instances to a DB handle.
@@ -229,7 +245,8 @@ def _downloadAll(session: Session, dbHandle, pmids: list, unique: bool=True) ->
     :param unique: if ``True``, only VersionID == "1" records are handled.
     """
     parser = PubMedXMLParser(unique)
-    pmid_sets = [pmids[100 * i:100 * i + 100] for i in range(len(pmids) // 100 + 1)]
+    pmid_sets = [pmids[100 * i:100 * i + 100]
+                 for i in range(len(pmids) // 100 + 1)]
     downloads = map(Download, pmid_sets)
     instances = map(parser.parse, downloads)
     streaming = partial(_streamInstances, session, dbHandle)
